/* ppm2rgba 1/1/97 Monty */

#include <stdio.h>

int main(int argc,char *argv[]){
  FILE *outfile=stdout;
  FILE *infile=stdin;

  if(argc>1){
    infile=fopen(argv[1],"rb");
    if(infile==NULL){
      fprintf(stderr,"ppm2rgba: Unable to open input file.\n");
      exit(-1);
    }
  }

  /* We only know 'P3' through 'P6' variations */
  {
    int x,y,depth,xi,yi;
    int type,temp;


    if(fgetc(infile)!='P'){
      fprintf(stderr,"ppm2rgba: input is not a PPM file.\n");
      exit(-1);
    }

    type=fgetc(infile)-48;
    fgetc(infile);

    while((temp=fgetc(infile))=='#'){
      fscanf(infile,"%*[^\n]s\n");
      fgetc(infile);
    }
    ungetc(temp,infile);
      
    switch(type){
    case 3:case 5:case 6:
      fscanf(infile,"%d",&x);
      fscanf(infile,"%d",&y);
      fscanf(infile,"%d%*1c",&depth);

      if(depth!=255){
	fprintf(stderr,"ppm2rgba: Input file is not 24 bit; this ppm2rgba\n"
		       "          version can only handle 24 bit P3/5/6 "
		"files.\n");
	exit(-1);
      }
      break;
    case 4:
      fscanf(infile,"%d",&x);
      fscanf(infile,"%d%*1c",&y);
      depth=1;

      break;
    default:
      fprintf(stderr,"ppm2rgba: input is an unknown variant (#%d) of PPM\n"
	             "          (ppm2rgba only knows variants 3 through 6).\n",
	      type);
      exit(-1);
    }

    fprintf(outfile,"RGBA32 %d %d\n",x,y);

    switch(type){
      int r,g,b;
    case 3:
      /* ASCII 24 bit */
      for(yi=0;yi<y;yi++){
	for(xi=0;xi<x;xi++){
	  fscanf(infile,"%d %d %d",&r,&g,&b);
	  fprintf(outfile,"%c%c%c%c",(unsigned char)r,
		  (unsigned char)g,
		  (unsigned char)b,
		  (unsigned char)255);
	}
      }
      break;
    case 5:
      /* Raw 8 bit grayscale */
      for(yi=0;yi<y;yi++){
	for(xi=0;xi<x;xi++){
	  r=g=b=(unsigned char)fgetc(infile);
	  fprintf(outfile,"%c%c%c%c",(unsigned char)r,
		  (unsigned char)g,
		  (unsigned char)b,
		  (unsigned char)255);
	}
      }
      break;
    case 6:
      /* Raw 24 bit RGB */
      for(yi=0;yi<y;yi++){
	for(xi=0;xi<x;xi++){
	  r=(unsigned char)fgetc(infile);
	  g=(unsigned char)fgetc(infile);
	  b=(unsigned char)fgetc(infile);
	  fprintf(outfile,"%c%c%c%c",(unsigned char)r,
		  (unsigned char)g,
		  (unsigned char)b,
		  (unsigned char)255);
	}
      }
      break;
    case 4:
      /* Raw bitpacked single bit-plane */
      for(yi=0;yi<y;yi++){
	int ti;
	for(xi=0;xi<x;xi+=8){
	  r=(unsigned char)fgetc(infile);
	  for(ti=0;(ti<8) && (xi+ti<x);ti++){
	    if(0x80 & (r<<ti))
	      fprintf(outfile,"%c%c%c%c",(unsigned char)0,
		  (unsigned char)0,
		  (unsigned char)0,
		  (unsigned char)255);
	    else
	      fprintf(outfile,"%c%c%c%c",(unsigned char)255,
		  (unsigned char)255,
		  (unsigned char)255,
		  (unsigned char)255);
	  }
	}
      }
      break;
    }
  }
  fclose(infile);
  fclose(outfile);
  return(0);
}    





