/* rgbatrans 1/1/97 Monty */

/* specify either an X RGB triplet (#xxyyzz), a color 'appearence' entry
   like transgif or a negative number for all pixels.  Last optional arg is 
   transparency level (0.0[trans]-1.0[opq] or 0%-100%) */

#define HS 256*256
#define hash(x) ((((x)<<4)&0xffffL)^(((x)>>12)&0xffffL))
#define R(x) (((x)>>16)&0xffL)
#define G(x) (((x)>>8)&0xffL)
#define B(x) ((x)&0xffL)

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <malloc.h>
#include <limits.h>

typedef struct hashentry{
  long index;
  long key;
  struct hashentry *next;
} hashentry;

hashentry *do_hash(hashentry *table,long key,int addp){
  int tag=hash(key);
  table+=tag;
  while(table->next){
    if(table->key==key)
      return(table);

    if(table->next==NULL){
      if(addp){
	table->next=calloc(1,sizeof(hashentry));
	table->next->index=-1;
	table->next->key=key;
	return(table->next);
      }else
	return(NULL);
    }
    table=table->next;
  }
  if(addp){
    table->index=-1;
    table->key=key;
    return(table); 
  }
  return(NULL);
}

int main(int argc,char *argv[]){
  FILE *outfile=stdout;
  FILE *infile=stdin;
  int i=1,x,y;
  int trans=0;
  float level=0;
  char test[8];

  /* First arg could be a filename */

  if(argc>1){
    if(!isdigit(argv[i][0]) && argv[i][0]!='#' && argv[i][0]!='-' &&
       argv[i][0]!='.'){
      infile=fopen(argv[i],"rb");
      if(infile==NULL){
	fprintf(stderr,"rgbatrans: Unable to open input file.\n");
	exit(-1);
      }
      i++;
      
    }
  }

  fscanf(infile,"%7c",test);
  test[7]='\0';
  if(strcmp(test,"RGBA32 ")){
    fprintf(stderr,"rgbatrans: input is not an RGBA32 file.\n");
    exit(-1);
  }
  
  fscanf(infile,"%d %d%*1c",&x,&y);

  {
    int xi,yi;
    long *c,*a;
    long *cP,*aP;
    int ptr;

    c=cP=malloc(x*y*sizeof(long));
    a=aP=malloc(x*y*sizeof(long));

    for(yi=0;yi<y;yi++){
      ptr=x*yi;
      for(xi=0;xi<x;xi++){
	c[ptr]=fgetc(infile)<<16L;
	c[ptr]|=fgetc(infile)<<8L;
	c[ptr]|=(unsigned char)fgetc(infile);
	a[ptr]=(unsigned char)fgetc(infile);
	ptr++;
      }
    }

    /* Do we need to count into the colormap? */

    if(argc>i && argv[i][0]=='#')
      trans=strtol(argv[i++]+1,NULL,16);
    else{

      hashentry *hashtable=calloc(HS,sizeof(hashentry)),*key;
      long colors=0;

      if(argc>i)
	trans=atoi(argv[i++]);

      if(trans>=0){
	for(yi=0;yi<y;yi++){  
	  c=cP+yi*x;
	  for(xi=0;xi<x;xi++){
	    key=do_hash(hashtable,c[xi],1);
	    if(key->index==-1){
	      if (colors==trans)
		break;
	      key->index=colors++;
	    }
	  }
	  if (colors==trans)
	    break;
	}
	if(colors!=trans){
	  fprintf(stderr,"rgbatrans: Given color index > number of colors in "
		  "image.\n");
	  exit(-1);
	}else
	  trans=key->key;
      }
    }

    fprintf(outfile,"RGBA32 %d %d\n",x,y);

    if(argc>i)
      if(argv[i][strlen(argv[i])-1]=='%')
	level=(atof(argv[i++]))/100;
      else
	level=atof(argv[i++]);

    for(yi=0;yi<y;yi++){
      c=cP+yi*x;
      a=aP+yi*x;
      for(xi=0;xi<x;xi++){
	if(trans<0 || c[xi]==trans){

	  fprintf(outfile,"%c%c%c%c",(unsigned char)R(c[xi]),
		  (unsigned char)G(c[xi]),
	          (unsigned char)B(c[xi]),
		  (unsigned char)(level*255));

	}else{

	  fprintf(outfile,"%c%c%c%c",(unsigned char)R(c[xi]),
		  (unsigned char)G(c[xi]),
	          (unsigned char)B(c[xi]),
		  (unsigned char)a[xi]);
	  
	}
      }
    }
  }
  fclose(infile);
  fclose(outfile);
  return(0);
}    






